"""Tests for the sanitize function."""
from sanitize_filename import sanitize


def test_invalid_chars():
    """Make sure invalid characters are removed in filenames."""
    assert(sanitize("A/B/C") == "ABC")
    assert(sanitize("A*C.d") == "AC.d")


def test_invalid_suffix():
    """Dots are not allowed at the end."""
    assert(sanitize("def.") == "def")
    assert(sanitize("def.ghi") == "def.ghi")


def test_reserved_words():
    """Make sure reserved Windows words are prefixed."""
    assert(sanitize("NUL") == "__NUL")
    assert(sanitize("..") == "__")


def test_long_names():
    """Make sure long names are truncated."""
    assert(len(sanitize("X" * 300)) == 255)



"""A permissive filename sanitizer."""
import unicodedata


def sanitize(filename):
    """Return a fairly safe version of the filename.

    We don't limit ourselves to ascii, because we want to keep municipality
    names, etc, but we do want to get rid of anything potentially harmful,
    and make sure we do not exceed Windows filename length limits.
    Hence a less safe blacklist, rather than a whitelist.
    """
    blacklist = ["\\", "/", ":", "*", "?", "\"", "<", ">", "|", "\0"]
    reserved = [
        "CON", "PRN", "AUX", "NUL", "COM1", "COM2", "COM3", "COM4", "COM5",
        "COM6", "COM7", "COM8", "COM9", "LPT1", "LPT2", "LPT3", "LPT4", "LPT5",
        "LPT6", "LPT7", "LPT8", "LPT9",
    ]  # Reserved words on Windows
    filename = ''.join(c for c in filename if c not in blacklist)
    filename = unicodedata.normalize('NFKD', filename)
    filename = filename.rstrip(". ")  # Windows does not allow these at end
    filename = filename.strip()
    if all([x == "." for x in filename]):
        filename = "__" + filename
    if filename in reserved:
        filename = "__" + filename
    if len(filename) == 0:
        filename = "__"
    if len(filename) > 255:
        filename = filename[:255]
        filename = filename.rstrip(". ")  # Re-check last character
    return filename



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sanitize_filename",
    version="1.0.0-dev1",
    author="Leo Wallentin | J++ Stockholm",
    author_email="mejl@leowallentin.se",
    description="A permissive filename sanitizer.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/jplusplus/sanitize-filename",
    packages=setuptools.find_packages(),
    python_requires='~=3.7',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
